<?php
    if ( !defined('K_COUCH_DIR') ) die(); // cannot be loaded directly
    
    class EmailEx{
	function validate( $field, $args ){
	    global $FUNCS, $__FILES;
	    
	    $input_name = $field->name . '_ex'; // By convention the associated 'file' input has an '_ex' appended. 
	    
	    //if( isset($_FILES[$input_name]) && !is_null($_FILES[$input_name]['tmp_name']) ){
	    if( isset($_FILES[$input_name]['tmp_name']) && @is_uploaded_file($_FILES[$input_name]['tmp_name']) ){
		$oFile = $_FILES[$input_name] ;
		
		// Check for POST errors in uploading
		if( $oFile['error'] !== UPLOAD_ERR_OK ){
		    switch( $oFile['error'] ){ 
			case UPLOAD_ERR_INI_SIZE: 
			    $err_msg = 'The uploaded file exceeds the upload_max_filesize directive in php.ini';
			    break;
			case UPLOAD_ERR_FORM_SIZE: 
			    $err_msg = 'The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form';
			    break;
			case UPLOAD_ERR_PARTIAL: 
			    $err_msg = 'The uploaded file was only partially uploaded';
			    break;
			case UPLOAD_ERR_NO_FILE:
			    if( $field->required ){
				$err_msg = 'No file was uploaded';
			    }
			    else{
				return;
			    }
			    break;
			case UPLOAD_ERR_NO_TMP_DIR: 
			    $err_msg = 'Missing a temporary folder';
			    break;
			case UPLOAD_ERR_CANT_WRITE: 
			    $err_msg = 'Failed to write file to disk';
			    break;
			case UPLOAD_ERR_EXTENSION: 
			    $err_msg = 'File upload stopped by extension';
			    break;
			default: 
			    $err_msg = 'Unknown upload error'; 
		    }
		    if( $err_msg ) return KFuncs::raise_error( $err_msg );
		}
		
		// Parse supplied parameters
		// e.g. "max_size:512&allowed_ext:jpeg,jpg,gif,png"
		$params = array();
		$arr_params = array_map( "trim", explode( '&', $args ) );
		foreach( $arr_params as $param ){
		    if( $param ){
			$arr_elems = array_map( "trim", explode( ':', $param ) );
			$params[$arr_elems[0]] = $arr_elems[1];
		    }
		}
		
		// Check if file of proper size
		$max_size = EmailEx::max_upload_size();
		if( isset($params['max_size']) && $FUNCS->is_non_zero_natural($params['max_size']) ){
		    $max_size = $params['max_size'];
		}
		$size_uploaded_file = $oFile['size']/1024; // bytes to KB
		if( $size_uploaded_file > $max_size ){
		    return KFuncs::raise_error( 'The uploaded file exceeds the '.$max_size.' KB permitted' );
		}
		
		// Check if extension is allowed
		$allowed_extensions = array();
		$arr_ext = array_map( "trim", explode( ',', $params['allowed_ext'] ) );
		foreach( $arr_ext as $ext ){
		    if( $ext ) $allowed_extensions[]=$ext;
		}
		if( !count($allowed_extensions) ) $allowed_extensions = EmailEx::allowed_extensions();
		$filename = $oFile['name'];
		$ext = strtolower( substr($filename, (strrpos($filename, '.') + 1)) ) ;
		if( !in_array($ext, $allowed_extensions) ){
		    return KFuncs::raise_error( 'File extension not allowed' );
		}
		
		// if image file, check if really so
		// TODO
		
		// Validation complete. Store info for further processing by send_mail
		if( !is_array($__FILES) ) $__FILES = array();
		$__FILES[$field->name] = $oFile;
	    }
	    else{
		return KFuncs::raise_error( "Input of file type named '".$input_name."' not found" );
	    }
	}
	
	function send_mail( $from, $to, $subject, $message='' ){ 
	    global $FUNCS, $__FILES;
	    
	    // sanitize inputs
	    $from = $FUNCS->_rsc( trim($from) );
	    $to = $FUNCS->_rsc( trim($to) );
	    $subject = $FUNCS->_rsc( trim($subject) );
	    
	    
	    // do we have attachments
	    $arr_uploads = array();
	    if( is_array($__FILES) ) $arr_uploads = $__FILES;
	    
	    $message = $FUNCS->unhtmlentities( $message, K_CHARSET ); // resurrecting (decoding) the entities. Shouldn't be required in HTML mails.
	    if( !count($arr_uploads) ){ // if no uploaded attachments, send simple mail
		$message = strip_tags( $message );
		return @mail( $to, $subject, $message, 'From: '.$from );
	    }
	    
	    // prepare multi-part mail
	    $uid = md5( uniqid(time()) );
	    $message = $FUNCS->_rsc( nl2br($message) ); //FIXIT: what is message is already HTML?
	    
	    $header = "From: ".$from."\r\n";
	    $header .= "Reply-To: ".$from."\r\n";
	    $header .= "MIME-Version: 1.0\r\n";
	    $header .= "Content-Type: multipart/mixed; boundary=\"".$uid."\"\r\n\r\n";
        
	    $msg .= "This is a multi-part message in MIME format.\r\n";
	    $msg .= "--".$uid."\r\n";
	    $msg .= "Content-type:text/html; charset=".K_CHARSET."\r\n";
	    $msg .= "Content-Transfer-Encoding: 7bit\r\n\r\n";
	    $msg .= $message."\r\n\r\n";
	    $msg .= "--".$uid;
	    foreach( $arr_uploads as $upload ){
		$file = $upload['tmp_name'];
		$file_size = filesize( $file );
		$filename = basename( $upload['name'] );
		$handle = fopen( $file, "rb" );
		$content = fread( $handle, $file_size );
		fclose( $handle );
		$content = chunk_split( base64_encode($content) );
		
		$msg .= "\r\n";
		$msg .= "Content-Type: application/octet-stream; name=\"".$filename."\"\r\n"; 
		$msg .= "Content-Transfer-Encoding: base64\r\n";
		$msg .= "Content-Disposition: attachment; filename=\"".$filename."\"\r\n\r\n";
		$msg .= $content."\r\n\r\n";
		$msg .= "--".$uid;
	    }
	    $msg .= "--";
	    if( !@mail($to, $subject, $msg, $header) ){
		echo 'Failed to send email';
	    }
	}
	
	function max_upload_size(){
	    return 512;
	}
	
	function allowed_extensions(){
	    return array('7z', 'aiff', 'asf', 'avi', 'bmp', 'csv', 'doc', 'fla', 'flv', 'gif', 'gz', 'gzip', 'jpeg', 'jpg', 'mid', 'mov', 'mp3', 'mp4', 'mpc', 'mpeg', 'mpg', 'ods', 'odt', 'pdf', 'png', 'ppt', 'pxd', 'qt', 'ram', 'rar', 'rm', 'rmi', 'rmvb', 'rtf', 'sdc', 'sitd', 'swf', 'sxc', 'sxw', 'tar', 'tgz', 'tif', 'tiff', 'txt', 'vsd', 'wav', 'wma', 'wmv', 'xls', 'xml', 'zip') ;
	}	
	
    }// end class
    